#include <app/Application.h>
#include <interface/Alert.h>
#include <InterfaceKit.h>
#include <storage/Entry.h>
#include <translation/TranslationUtils.h>
#include <stdio.h>

#include "Wind.h"
//#include "texture.h"

#define MENU_HEIGHT 15

// ----------------------------------------------------------------------
// OpenGL drawing thread

void ShowMsg(char *msg)
{
	BAlert *alert = new BAlert("Message",msg,"OK");
	alert->Go();
}

void DrawAxes()
{
	/////////// Y /////////////
	glBegin(GL_QUADS);
		glColor3f(1,0,0);
		glVertex3f(0,0,0);
		glVertex3f(0,100,0);
		glVertex3f(1,100,0);
		glVertex3f(1,0,0);
	
		/////////// X /////////////
		glColor3f(0,1,0);
		glVertex3f(0,0,0);
		glVertex3f(100,0,0);
		glVertex3f(100,1,0);
		glVertex3f(0,1,0);
	
		/////////// Z /////////////
		glColor3f(0,0,1);
		glVertex3f(0,0,0);
		glVertex3f(0,0,100);
		glVertex3f(1,0,100);
		glVertex3f(1,0,0);
	glEnd();
}

int32 Wind::render_thread( void *data )
{
	Wind *wnd = (Wind *)data;

	wnd->MakeCurrent();
	
	if(wnd->texture_loaded)
	{
		if(!wnd->InitTexture()) wnd->texture_enable = false;
		else wnd->texture_enable = true;
	}
	
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	//glOrtho( -110, 110, -110, 110, -120, 120 );
	glOrtho( -1310, 1310, -1310, 1310, -1320, 1320 );
	glMatrixMode( GL_MODELVIEW );

	glClearColor( 0.f, 0.1f, 0.f, 1.f );
	
	// Enable Z-BUFFER
	glEnable( GL_DEPTH_TEST );
	// Enable TEXTURing
	if(wnd->texture_enable)
	{
		glEnable( GL_TEXTURE_2D );
		glBindTexture( GL_TEXTURE_2D, wnd->texture[0] );
	}
	glShadeModel( GL_SMOOTH );
	
	while( !wnd->window_teardown )
	{
	glDisable(GL_TEXTURE_2D);
	DrawAxes();
		
	glEnable(GL_TEXTURE_2D);
	
	wnd->DrawObject(&wnd->object, wnd->texture[0]);		
		glRotatef( 0.5, 1.0, 1.0, 0.0 );
	// 	glTranslatef(0.05,0,0);
		

		
		glBegin(GL_QUADS);
						glTexCoord2f(0,1);
					glColor3f(1,0,0);
			glVertex3f(-20,20,0);
						glTexCoord2f(1,1);
					glColor3f(0,1,0);
			glVertex3f(20,20,0);
						glTexCoord2f(1,0);
					glColor3f(0,0,1);
			glVertex3f(20,-20,0);
						glTexCoord2f(0,0);
					glColor3f(1,0,1);
			glVertex3f(-20,-20,0);
		glEnd();

		// use DOUBLE BUFFER. SYNC = TRUE (enable)
		wnd->SwapBuffers( wnd->vsync );
		glDrawBuffer( GL_BACK );

		wnd->YieldCurrent();

		// clear COLOR and Z-BUFFER
		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	}
	
	wnd->ReleaseCurrent();
}

void Wind::DeviceInfo( uint32 device_id, uint32 monitor, const char *name, bool depth, bool stencil, bool accum )
{
	char buf[256];
	BMessage msg ('addm');
	msg.AddInt32("num",device_id);
	msg.what = 'dev';
	sprintf( buf, "%s  mon=%x", name, monitor );
	BMenuItem *item = new BMenuItem( buf, new BMessage(msg));
	DeviceMenu->AddItem( item );
	if( device_id == BGL_DEVICE_SOFTWARE )
	{
		item->SetMarked( true );
		checkedDevice = item;
	}
	
	currentDevice++;
	devices[currentDevice].deviceID = device_id;
	devices[currentDevice].first = 0;
}


// ----------------------------------------------------------------------
// Constructor and destructor

Wind::Wind( const char *title )
	: BDirectGLWindow( BRect( 100.0, 100.0, 499.0, 499.0 ), title,
			   		 B_TITLED_WINDOW_LOOK, B_NORMAL_WINDOW_FEEL,
			   		 B_ASYNCHRONOUS_CONTROLS | B_NOT_ZOOMABLE )
{
	window_teardown = false;	// drawing thread control variable
	currentDevice = -1;			// current DEVICE number
	draw_thread = -1;			// drawing thread ID
	initialized = false;
	vsync = true;
	texture_bits = NULL;

	// Create a menu
	BRect r(0,0,0,MENU_HEIGHT);
	BMenuBar *mb;
	mb = new BMenuBar(r,"main menu");
	AddChild( mb );
	BMessage msg ('addm');

	msg.ReplaceInt32("num",256);
	msg.what = 'dev';
	mb->AddItem( DeviceMenu = new BMenu("Device") );
	mb->ResizeToPreferred();
	
	texture_bits = LoadTexture("/boot/home/simpleGLlesson2/tex1.png");
	if(texture_bits == NULL) texture_loaded = false;
		else texture_loaded = true;
		
	if(!LoadObject("/boot/home/simpleGLlesson2/meshes/CASTLE3.3D", &object)) ShowMsg("Error loading object");
	
	// Gets Available Devices
	EnumerateDevices( BGL_MONITOR_ANY, BGL_FASTEST | BGL_DOUBLE, BGL_ANY, BGL_NONE, BGL_NONE );

	// Restarts OPENGL
	if( RestartGL( BGL_DEVICE_SOFTWARE ) < B_OK ) return;
	
	Show();
	
	ShowMsg("Use SPACE to En/Disable VSYNC");
}

Wind::~Wind( void )
{
	status_t ret;

	Hide();
	Sync();
	window_teardown = true;
	wait_for_thread( draw_thread, &ret );
	
	if(texture_bits != NULL) delete texture_bits;
}


status_t Wind::RestartGL( uint32 deviceID )
{
	status_t ret;

	window_teardown = true;
	if( draw_thread >= 0 )
		wait_for_thread( draw_thread, &ret );

	if( initialized )
	{
		ShutdownGL();
		initialized = false;
	}
	initialized = true;

	window_teardown = false;
	
	// Start the drawing thread.
	draw_thread = spawn_thread( render_thread, "OpenGL drawing thread", B_DISPLAY_PRIORITY, this );
	if( draw_thread < 0 )
	{
		be_app->PostMessage( B_QUIT_REQUESTED );
		return B_ERROR;
	}

	fullscreenEnabled = false;
	if( InitializeGL( deviceID, BGL_FASTEST | BGL_DOUBLE, BGL_ANY, BGL_NONE, BGL_NONE ) < B_OK )
	{
		be_app->PostMessage( B_QUIT_REQUESTED );
		exit(0);  // Fix me
	}
	
	BMessage msg ('addm');
	msg.AddInt32("XxY",0);
	msg.what = 'res';
	int32 ct;
	for( ct=0; ct<MAX_DEVICES; ct++ )
	{
		if( devices[ct].deviceID == deviceID )
			break;
	}

	resume_thread( draw_thread );
	return B_OK;
}


bool Wind::QuitRequested( void )
{
	be_app->PostMessage( B_QUIT_REQUESTED );
	return false;
}

void Wind::MessageReceived( BMessage *msg )
{
	BMenuItem *i;
	switch( msg->what )
	{
	case 'dev':
		{
			msg->FindPointer("source", (void **)&i);
			checkedDevice->SetMarked( false );
			checkedDevice = i;
			checkedDevice->SetMarked( true );
	
			uint32 new_mode = msg->FindInt32( "num" );
			if( RestartGL( new_mode ) < B_OK )
				exit(0);
		}
		break;

	case B_KEY_DOWN:
		if( RestartGL( 1 ) < B_OK ) exit(0);
		
		vsync = !vsync;
		
		if( fullscreenEnabled )
		{
			fullscreenEnabled = false;
			FullscreenDisable();
			ResizeTo( fullscreenWidth-1, fullscreenHeight-1 );
			break;
		}
		paused = !paused;
	
	default:
		BDirectWindow::MessageReceived( msg );
		break;
	}
}


void Wind::FrameResized( float width, float height )
{
	MakeCurrent();
	if( fullscreenEnabled )
	{
		glViewport( 0, 0, fullscreenWidth, fullscreenHeight );
	}
	else
	{
		if( height > MENU_HEIGHT )
			glViewport( 0, 0, int(width-1), int(height-1) - MENU_HEIGHT );
	}
	ReleaseCurrent();
}

BRect Wind::UpdateGLRect( BRect window_rect )
{
	BRect r;

	if( fullscreenEnabled )
	{
		r = BRect( 0, 0, fullscreenWidth-1, fullscreenHeight-1 );
	}
	else
	{
		r = window_rect;
		if( r.bottom > MENU_HEIGHT )
			r.top += MENU_HEIGHT;
		else
			r.bottom = 0;
	}
	return r;
}


/////////////////////////////////////////////////////////////////////////

BBitmap *Wind::LoadTexture( const char *tex_file )
{
	BEntry entry(tex_file);
	entry_ref ref;
	
	if(entry.GetRef(&ref) != B_OK) return NULL;

	BBitmap *bits = BTranslationUtils::GetBitmap( &ref );
	if( bits == NULL ) return NULL;

	if( bits->ColorSpace() != B_RGBA32 && bits->ColorSpace() != B_RGB32 )
	{
    	delete bits; return NULL;
    }

	bits->Lock();

	int32 bpr = bits->BytesPerRow();

	char *top = static_cast<char *>( bits->Bits() );
	char *bottom = top + bits->Bounds().IntegerHeight() * bpr;

	char *tmp_bytes = new char[bpr];
	while( top < bottom )
	{
		memcpy( tmp_bytes, bottom, bpr );

#if 0
		memcpy( bottom, top, bpr );
		memcpy( top, tmp_bytes, bpr );
#else
		rgb_color *b = (rgb_color *)( bottom );
		rgb_color *t = (rgb_color *)( top );
		for( int i = 0; i < ( bpr / 4 ); i++ )
		{
			b->red = t->blue;
			b->green = t->green;
			b->blue = t->red;
			b->alpha = t->alpha;
			b++, t++;
		}
		t = (rgb_color *)( top );
		b = (rgb_color *)( tmp_bytes );
		for( int i = 0; i < ( bpr / 4 ); i++ )
		{
			t->red = b->blue;
			t->green = b->green;
			t->blue = b->red;
			t->alpha = b->alpha;
			b++, t++;
		}
#endif
		bottom -= bpr;
		top += bpr;
	}

	bits->Unlock();

	return bits;
}

///////////////////////////////////////////////////////////////////////////

bool Wind::InitTexture()
{
	if( texture_bits == NULL ) return false;

	glGenTextures( 1, &texture[0] );
	glBindTexture( GL_TEXTURE_2D, texture[0] );
	
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );

	if( texture_bits->ColorSpace() == B_RGBA32 ) {
		glTexImage2D( GL_TEXTURE_2D, 0, 
					  4, 
					  texture_bits->Bounds().IntegerWidth() + 1,
					  texture_bits->Bounds().IntegerHeight() + 1,
					  0, GL_RGBA, GL_UNSIGNED_BYTE,
					  texture_bits->Bits() );
	} else if( texture_bits->ColorSpace() == B_RGB32 ) {
		glTexImage2D( GL_TEXTURE_2D, 0, 
					  3, 
					  texture_bits->Bounds().IntegerWidth() + 1,
					  texture_bits->Bounds().IntegerHeight() + 1,
					  0, GL_RGBA, GL_UNSIGNED_BYTE,
					  texture_bits->Bits() );
	} else {
		return false;
	}
	
	return true;
}

bool Wind::LoadObject(const char * object_file, OBJECT* obj)
{
	int num_faces, i;
	FILE* fp = fopen(object_file, "rt");
		
	if(!fp) return false;
	
	fscanf(fp,"%d",&num_faces);
	
	obj->face = new FACE[num_faces];
	if(obj->face == NULL) return false;
	obj->num_faces = num_faces;
	
	for(i=0; i<num_faces; i++)
	{
		fscanf(fp,"%f %f %f %f %f %f %f %f %f",&obj->face[i].v1.x, &obj->face[i].v1.y, &obj->face[i].v1.z, &obj->face[i].v2.x, &obj->face[i].v2.y, &obj->face[i].v2.z, &obj->face[i].v3.x, &obj->face[i].v3.y, &obj->face[i].v3.z);
	}
	
	fclose(fp);
	
	return true;
}

void Wind::DrawObject(OBJECT* obj, GLuint texture)
{
	int i, num_faces;
	
	num_faces = obj->num_faces;
	
	glBindTexture(GL_TEXTURE_2D, texture);
	
	glBegin(GL_TRIANGLES);
	for(i=0; i<num_faces; i++)
	{
				glTexCoord2f(1,0);
			glColor3f(1,0,0);
		glVertex3f(obj->face[i].v1.x, obj->face[i].v1.y, obj->face[i].v1.z);
				glTexCoord2f(0,0);
			glColor3f(0,1,0);
		glVertex3f(obj->face[i].v2.x, obj->face[i].v2.y, obj->face[i].v2.z);
				glTexCoord2f(1,0); glTexCoord2f(0,1);
			glColor3f(0,0,1);
		glVertex3f(obj->face[i].v3.x, obj->face[i].v3.y, obj->face[i].v3.z);
	}
	glEnd();
}