/////// VARIABLES ///////////
GLuint texture[1];
BBitmap *texture_bits;
bool texture_loaded = false;

/////// FUNCTIONS ///////////
BBitmap *LoadTexture( const char *tex_file );
bool InitTexture(const char *tex_file);

/////////////////////////////////////////////////////////////////////////

BBitmap *LoadTexture( const char *tex_file )
{
	BEntry entry(tex_file);
	entry_ref ref;
	
	if(entry.GetRef(&ref) != B_OK) return NULL;

	BBitmap *bits = BTranslationUtils::GetBitmap( &ref );
	if( bits == NULL ) return NULL;

	if( bits->ColorSpace() != B_RGBA32 && bits->ColorSpace() != B_RGB32 )
	{
    	delete bits; return NULL;
    }

	bits->Lock();

	int32 bpr = bits->BytesPerRow();

	char *top = static_cast<char *>( bits->Bits() );
	char *bottom = top + bits->Bounds().IntegerHeight() * bpr;

	char *tmp_bytes = new char[bpr];
	while( top < bottom ) {
		memcpy( tmp_bytes, bottom, bpr );

#if 0
		memcpy( bottom, top, bpr );
		memcpy( top, tmp_bytes, bpr );
#else
		rgb_color *b = (rgb_color *)( bottom );
		rgb_color *t = (rgb_color *)( top );
		for( int i = 0; i < ( bpr / 4 ); i++ ) {
			b->red = t->blue;
			b->green = t->green;
			b->blue = t->red;
			b->alpha = t->alpha;
			b++, t++;
		}
		t = (rgb_color *)( top );
		b = (rgb_color *)( tmp_bytes );
		for( int i = 0; i < ( bpr / 4 ); i++ ) {
			t->red = b->blue;
			t->green = b->green;
			t->blue = b->red;
			t->alpha = b->alpha;
			b++, t++;
		}
#endif
		bottom -= bpr;
		top += bpr;
	}

	bits->Unlock();

	return bits;
}

///////////////////////////////////////////////////////////////////////////

bool InitTexture(const char *tex_file)
{
	texture_bits =  NULL;
	
	texture_bits = LoadTexture( tex_file );
	if( texture_bits == NULL ) return false;//

	glGenTextures( 1, &texture[0] );
	glBindTexture( GL_TEXTURE_2D, texture[0] );
	
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );

	if( texture_bits->ColorSpace() == B_RGBA32 ) {
		glTexImage2D( GL_TEXTURE_2D, 0, 
					  4, 
					  texture_bits->Bounds().IntegerWidth() + 1,
					  texture_bits->Bounds().IntegerHeight() + 1,
					  0, GL_RGBA, GL_UNSIGNED_BYTE,
					  texture_bits->Bits() );
	} else if( texture_bits->ColorSpace() == B_RGB32 ) {
		glTexImage2D( GL_TEXTURE_2D, 0, 
					  3, 
					  texture_bits->Bounds().IntegerWidth() + 1,
					  texture_bits->Bounds().IntegerHeight() + 1,
					  0, GL_RGBA, GL_UNSIGNED_BYTE,
					  texture_bits->Bits() );
	} else {
		return false;
	}
	
	texture_loaded = true;
	return true;
}